"""
Echoview Code Operator source file
========================================================================================

Created by Echoview® 13.1.113

See the Echoview help file for Code operator documentation and examples.

NumPy User Guide is available from here: https://docs.scipy.org/doc/numpy/user/
NumPy Reference is available from here: https://docs.scipy.org/doc/numpy/reference/
SciPy Reference Guide is available from here: https://docs.scipy.org/doc/scipy/reference/
"""

# Authorship information
__author__ = "Echoview Software Pty Ltd"
__copyright__ = "Copyright 2023"

# System Imports
from typing import List

# Libraries
import echoview as ev
import numpy as np


class Operator(ev.OperatorBase):

    """Replicate Echoview's XxY statistic operator's Mean computation,
    along the Columns (pings) axis of an echogram, then compare the
    difference between the XxY statistic operator computation.

    Operands
    ----------------------------------------------------------------------------------------

    * Operand 1 - Single beam data in dB.
    * Operand 2 - XxY statistic operator.

    Notes
    ----------------------------------------------------------------------------------------

    Compute the average of the ping sample data. The number of pings
    to average is specified by Window size (pings) on the Code page of
    the Variable properties dialog box.

    """

    # Refer to the __init__ example in appendix B.2.2 "Programming an
    # attribute for the Code operator" to set up this method.
    def __init__(self):
        '''Threshold of 1e-10 db.'''

    # Refer to section 7.2 "Choosing a data type with the result_type
    # method".
    def '''result_type'''
        return '''Boolean data type from the MeasurementType class.'''

    def eval(self, inputs: List[ev.OperandInput]):

        # Refer to section 2.1.2 "About the commands in the default
        # Echoview Python source file" to assign Operand 1 to the
        # variable first_input.
        first_input = inputs[0]

        # Refer to the example in section 13.2 "Ping sample data from
        # all pings in the window of measurement", to use list
        # comprehension to collate the ping sample data. Then try to
        # work out the shape of the ping_samples_db array. If you get
        # stuck, refer to section 13.3 "The shape of an array".
        ping_samples_db = [
            ping.data for ping in first_input.window_measurements]

        # The formula to convert from dB to linear is 10^(X/10), where
        # X is the ping sample data in dB.
        # Hint: Use the np.power and np.divide functions. (Attempt the
        # computation in a single line of code.)
        ping_samples_linear = np.power(10, np.divide(ping_samples_db, 10))

        # Compute the mean.
        # Hint: ping_samples_linear has shape (pings, range). Use the
        # axis parameter in the np.mean method to average the pings.
        # Refer to section 13.3 "The shape of an array" for the
        # syntax.
        mean_linear = np.mean(ping_samples_linear, axis=0)

        # The formula to convert from linear to dB is 10*log10(X), where
        # X is the ping sample data in linear.
        # Hint: Use the np.log10 and np.multiply functions. (Attempt the
        # computation in a single line of code.)
        mean_db = np.multiply(np.log10(mean_linear), 10)

        # This is the ping sample data from Operand 2.
        # Refer to section 5.4 "Returning the ping samples from another
        # operand".
        xxy_statistic_samples = '''Ping data samples from the "XxY statistic - Mean of pings" Echoview variable.'''

        # Hint: Use the np.subtract method to compute the difference
        # between mean_db and xxy_statistic_samples.
        difference = '''mead_db - xxy_statistic_samples'''

        # Hint: Use np.abs to test for the absolute magnitude of the
        # difference.
        # Refer to appendix D "Examples using numpy.where" for some
        # examples on using np.where.
        return np.where('''If the difference is less than the threshold.''', True, False)
