"""
Echoview Code Operator source file
========================================================================================

Created by Echoview® 13.1.113

See the Echoview help file for Code operator documentation and examples.

NumPy User Guide is available from here: https://docs.scipy.org/doc/numpy/user/
NumPy Reference is available from here: https://docs.scipy.org/doc/numpy/reference/
SciPy Reference Guide is available from here: https://docs.scipy.org/doc/scipy/reference/
"""

# Authorship information
__author__ = "Echoview Software Pty Ltd"
__copyright__ = "Copyright 2023"

# System Imports
from typing import List

# Libraries
import echoview as ev
import numpy as np


class Operator(ev.OperatorBase):

    """Create a circular mask for multibeam pings.

    Operands
    ----------------------------------------------------------------------------------------

    * Operand 1 - Multibeam data.
    """

    # Refer to the __init__ example in appendix B2.2 "Programming an
    # attribute" for the syntax for programming this method.
    def __init__(self):
        # Circular mask's coordinates (r_0, phi) and radius R.
        self.center_range = 6.0  # r_0 (meters)
        self.center_angle = -6  # phi (degrees)
        self.radius = 0.5  # R (meters)

    def eval(self, inputs):

        # Refer to section 2.1.2 "About the commands in the default
        # Echoview Python source file" to assign Operand 1 to the
        # variable first_input.
        first_input = inputs[0]

        # Refer to section 9.2 "Utilizing the window of measurements"
        # to get the matched ping.
        ping = first_input.measurement

        # Hint: Multibeam data has shape (range, beam). Use the
        # np.shape method to obtain the number of samples from the
        # range axis of the matched ping.
        # Refer to section 13.3 "The shape of an array".
        sample_count = ping.data.shape[0]

        # For a given beam, compute the range coordinate of each
        # sample as the midpoint of that sample's thickness.
        # Hint: Refer to the attributes of the Measurement class at
        # https://support.echoview.com/WebHelp/How_To/Use_The_Code_Operator/Using_the_Code_operator.htm#class_Measurement
        # to obtain the start and stop range of the ping.
        sample_thickness = (ping.stop_range-ping.start_range)/sample_count
        half_thickness = sample_thickness/2
        # Hint: Use half_thickness to traverse from a half-step
        # forward from the start range to a half-step backward from
        # the stop range.
        range_values_1D = np.linspace(
            ping.start_range+half_thickness, ping.stop_range-half_thickness, sample_count)

        # At a given range, obtain the angle coordinate of each
        # beam in radians.
        # Hint: Refer to the attributes of the Measurement class at
        # https://support.echoview.com/WebHelp/How_To/Use_The_Code_Operator/Using_the_Code_operator.htm#class_Measurement
        # to obtain the beam angles. Convert these from degrees to
        # radians.
        beam_angles_1D = np.radians(ping.beam_angle)

        # Create a lattice representing the coordinates (r_i, theta_i)
        # of each sample in the ping's fan.
        sample_ranges, sample_angles = np.meshgrid(
            range_values_1D, beam_angles_1D, indexing='ij', sparse=True)

        # Compute |r_0 - r_i| by finding the difference in the x and y
        # components.
        # Hints:
        # - First convert phi (from __init__) to radians.
        # - Then use the np.sin and np.cos methods to change all the
        #   coordinates from polar (r, theta) to cartesian (x, y).
        # - Finally use np.sqrt to compute the distance.
        center_angle_radians = np.radians(self.center_angle)
        distance_x = self.center_range*np.cos(center_angle_radians) \
            - sample_ranges*np.cos(sample_angles)
        distance_y = self.center_range*np.sin(center_angle_radians) \
            - sample_ranges*np.sin(sample_angles)
        distance_2D = np.sqrt(distance_x**2 + distance_y**2)

        # Set ping samples to no data when |r_0 - r_i| < R.
        # Hints:
        # - Use the np.where method. Refer to appendix D "Examples
        #   using numpy.where" for some examples on using np.where.
        # - np.nan is no data in Echoview.
        return np.where(distance_2D < self.radius, np.nan, ping.data)
